package detector.audio.processing.test;

import junit.framework.TestCase;
import detector.audio.processing.*;

import java.util.*;
import java.io.*;

import android.content.Context;
import android.test.ActivityInstrumentationTestCase2;
import android.test.AndroidTestCase;
import android.test.suitebuilder.annotation.Smoke;

/**
 * Testing the soundClass
 * @author Kyle
 *
 */
public class SoundClassTest extends AndroidTestCase {
	
	private static final String TEST_FILENAME = "testdata.dat";
	
	public SoundClassTest() {
		super();
	}
	
	/**
	 * Tests if it can get sounds
	 */
	public void testGetSounds() {
		SoundClass.clear();
		SoundClass sc = SoundClass.getSoundClass("Test1");
		SoundClass sc1 = SoundClass.getSoundClass("Test2");
		SoundClass sc2 = SoundClass.getSoundClass("Test3");
		Set<SoundClass> scs = new HashSet<SoundClass>();
		scs.add(sc);
		scs.add(sc1);
		scs.add(sc2);
		
		contains(SoundClass.getSounds(), scs);
	}
	
	private void contains(Collection<SoundClass> c1, Collection<SoundClass> c2) {
		for (SoundClass s : c1) {
			boolean c = false;
			for (SoundClass sc : c2) {
				if (s.equals(sc))
					c = true;
			}
			if (!c)
				fail("A soundclass was not in the other's set");
		}
	}
	
	
	//Tests if it can write and read from file
	public void testSaveLoad() {
		SoundClass.clear();
		
		SoundClass sc = SoundClass.getSoundClass("Test1");
		SoundClass sc1 = SoundClass.getSoundClass("Test2");
		SoundClass sc2 = SoundClass.getSoundClass("Test3");
		Set<SoundClass> scs = new HashSet<SoundClass>();
		scs.add(sc);
		scs.add(sc1);
		scs.add(sc2);
		
		try {
			FileOutputStream fos = getContext().openFileOutput(TEST_FILENAME, Context.MODE_PRIVATE);
			ObjectOutputStream oos = new ObjectOutputStream(fos);
			oos.writeObject(sc);
			oos.writeObject(sc1);
			oos.writeObject(sc2);
			oos.close();
			fos.close();
	
			FileInputStream fis = getContext().openFileInput(TEST_FILENAME);
			ObjectInputStream ois = new ObjectInputStream(fis);
			
			
			Set<SoundClass> scs2 = new HashSet<SoundClass>();
			scs2.add((SoundClass)ois.readObject());
			scs2.add((SoundClass)ois.readObject());
			scs2.add((SoundClass)ois.readObject());
			
			contains(scs2, scs);
		} catch (Exception ex) {
			fail("" + ex.getMessage());
		}
		contains(scs,SoundClass.getSounds());
	}
	
}
