package edu.washington.cs.sounddetector.server.api.exception;

/**
 * <p>
 * Base class for all exceptions thrown by server api calls. Subclass this if you need to provide an
 * exception on the client side for a failure case of a method.
 * </p>
 * 
 * <p>
 * Subclasses should have a descriptive name which clearly describes the error. They should provide
 * a single constructor which takes all the necessary data for a client to figure out exactly what
 * went wrong, and provide getter methods for this data. Also, make sure to call the super
 * constructor.
 * </p>
 * 
 * @author the.dylan.price@gmail.com
 */
public abstract class SoundDetectorAPIException extends Exception
{

    private static final long serialVersionUID = -2320458565807877689L;

    public SoundDetectorAPIException()
    {
        super();
    }

    public SoundDetectorAPIException(String message)
    {
        super(message);
    }

    public SoundDetectorAPIException(Throwable throwable)
    {
        super(throwable);
    }

    public SoundDetectorAPIException(String message, Throwable throwable)
    {
        super(message, throwable);
    }
}
