package edu.washington.cs.sounddetector.server.sound;

import static org.junit.Assert.assertArrayEquals;

import javax.sound.sampled.AudioFormat;

import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

public class SoundProcessorTest
{

    private static SoundProcessorBuilder builder;

    private AudioFormat format;
    private byte[] sound;
    private double[] samples;
    private double[] features;
    private String[] featureNames;
    private SoundProcessor processor;

    @BeforeClass
    public static void setUpClass() throws Exception
    {
        builder = new SoundProcessorBuilder()
        {
            {
                setWindowSize(3);
                setWindowOverlap(0.0);
                setSamplingRate(8000);
                setNormalize(false);

                setFeatures(new jAudioFeatureExtractor.AudioFeatures.PowerSpectrum(),
                        new jAudioFeatureExtractor.AudioFeatures.SpectralCentroid());

                setAggregators(new jAudioFeatureExtractor.Aggregators.Mean(),
                        new jAudioFeatureExtractor.Aggregators.StandardDeviation());
            }
        };
    }

    @Before
    public void setUp()
    {
        format = new AudioFormat(8000, 16, 1, true, false);
        sound = new byte[] { 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i' };
        samples = new double[] { 0.768585205078125, 0.784271240234375, 0.799957275390625,
                0.815643310546875 };
        features = new double[] { 0.17763568394002505, 0.09921274837688543, 0.17918242763122433,
                0.251214793389404, 0.14030801431490064, 0.25340221929501305 };
        featureNames = new String[] { "Power Spectrum Overall Average0",
                "Power Spectrum Overall Average1", "Spectral Centroid Overall Average0",
                "Power Spectrum Overall Standard Deviation0",
                "Power Spectrum Overall Standard Deviation1",
                "Spectral Centroid Overall Standard Deviation0" };
        processor = builder.build();
    }

    @Test
    public void testSamples()
    {
        double[] samples = processor.sample(sound, format);
        assertArrayEquals(this.samples, samples, 0.01);
    }

    @Test
    public void testExtractFeatures() throws Exception
    {
        double[] samples = processor.sample(sound, format);
        SoundFeatures soundFeatures = processor.extractFeatures(samples);
        assertArrayEquals(this.features, soundFeatures.getFlattenedFeatureValues(), 0.01);
        assertArrayEquals(this.featureNames, soundFeatures.getFlattenedFeatureNames());
    }
}
